<?php

declare(strict_types = 1);

namespace Drupal\file_extractor\Extractor;

use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Messenger\MessengerInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Plugin\PluginBase;
use Drupal\Core\StreamWrapper\StreamWrapperInterface;
use Drupal\Core\StreamWrapper\StreamWrapperManagerInterface;
use Psr\Log\LoggerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\Mime\MimeTypeGuesserInterface;

/**
 * Base class for FileExtractorExtractor plugins.
 *
 * @ingroup plugin_api
 */
abstract class ExtractorPluginBase extends PluginBase implements ExtractorPluginInterface, ContainerFactoryPluginInterface {

  /**
   * Stream wrapper manager service.
   *
   * @var \Drupal\Core\StreamWrapper\StreamWrapperManagerInterface
   */
  protected $streamWrapperManager;

  /**
   * Mime type guesser service.
   *
   * @var \Symfony\Component\Mime\MimeTypeGuesserInterface
   */
  protected $mimeTypeGuesser;

  /**
   * Logger.
   *
   * @var \Psr\Log\LoggerInterface
   */
  protected $logger;

  /**
   * Entity type manager service.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * Messenger service.
   *
   * @var \Drupal\Core\Messenger\MessengerInterface
   */
  protected $messenger;

  /**
   * {@inheritdoc}
   */
  public function __construct(
    array $configuration,
    $plugin_id,
    array $plugin_definition,
    StreamWrapperManagerInterface $stream_wrapper_manager,
    MimeTypeGuesserInterface $mime_type_guesser,
    LoggerInterface $logger,
    EntityTypeManagerInterface $entity_type_manager,
    MessengerInterface $messenger
  ) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->streamWrapperManager = $stream_wrapper_manager;
    $this->mimeTypeGuesser = $mime_type_guesser;
    $this->logger = $logger;
    $this->entityTypeManager = $entity_type_manager;
    $this->messenger = $messenger;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    // @phpstan-ignore-next-line
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('stream_wrapper_manager'),
      $container->get('file.mime_type.guesser'),
      $container->get('logger.channel.file_extractor'),
      $container->get('entity_type.manager'),
      $container->get('messenger')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return [];
  }

  /**
   * {@inheritdoc}
   */
  public function getConfiguration() {
    return $this->configuration;
  }

  /**
   * {@inheritdoc}
   */
  public function setConfiguration(array $configuration) : void {
    $this->configuration = $configuration + $this->defaultConfiguration();
  }

  /**
   * Form validation handler.
   *
   * @param array $form
   *   An associative array containing the structure of the plugin form as built
   *   by static::buildConfigurationForm().
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current state of the complete form.
   */
  public function validateConfigurationForm(array &$form, FormStateInterface $form_state) : void {
  }

  /**
   * Form submission handler.
   *
   * @param array $form
   *   An associative array containing the structure of the plugin form as built
   *   by static::buildConfigurationForm().
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current state of the complete form.
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state) : void {
    $this->setConfiguration($form_state->getValues());
  }

  /**
   * Helper method to get the real path from an uri.
   *
   * @param string $uri
   *   The URI of the file, e.g. public://directory/file.jpg.
   *
   * @return string
   *   The real path to the file if it is a local file. An URL otherwise.
   */
  protected function getRealpath(string $uri) : string {
    /** @var \Drupal\Core\StreamWrapper\StreamWrapperInterface $wrapper */
    $wrapper = $this->streamWrapperManager->getViaUri($uri);
    $scheme = $this->streamWrapperManager->getScheme($uri);
    $local_wrappers = $this->streamWrapperManager->getWrappers(StreamWrapperInterface::LOCAL);
    if (in_array($scheme, array_keys($local_wrappers))) {
      return $wrapper->realpath();
    }
    else {
      return $wrapper->getExternalUrl();
    }
  }

  /**
   * Helper method to get the PDF MIME types.
   *
   * @return array
   *   An array of the PDF MIME types.
   */
  protected function getPdfMimeTypes() : array {
    $pdf_mime_types = [];
    $pdf_mime_types[] = $this->mimeTypeGuesser->guessMimeType('dummy.pdf');
    return $pdf_mime_types;
  }

  /**
   * Check if the binary path is a path or only the binary name.
   *
   * @param string $binary_path
   *   The binary path.
   *
   * @return bool
   *   TRUE if only a binary name. FALSE otherwise.
   */
  protected function isBinaryName(string $binary_path) : bool {
    if ((strpos($binary_path, '/') === FALSE) && (strpos($binary_path, '\\') === FALSE)) {
      return TRUE;
    }

    return FALSE;
  }

  /**
   * Helper function to check that a binary and a file still exists.
   *
   * @param string $binary_path
   *   The binary path.
   * @param string $file_path
   *   The path to the file to extract.
   * @param array $log_variables
   *   Log variables.
   *
   * @return bool
   *   TRUE if binary and file exist. FALSE otherwise.
   */
  protected function checkBinaryAndFile(string $binary_path, string $file_path, array &$log_variables) : bool {
    $log_variables['@binary_path'] = $binary_path;
    $log_variables['@file_path'] = $file_path;

    // Test that the binary is still available.
    if (!$this->isBinaryName($binary_path) && !file_exists($binary_path)) {
      $this->logger->error('Extractor binary was not found at @binary_path.', $log_variables);
      return FALSE;
    }

    // Test that the file is still available.
    if (!file_exists($file_path)) {
      $this->logger->error('File was not found at @file_path.', $log_variables);
      return FALSE;
    }

    return TRUE;
  }

  /**
   * Helper function to run shell command.
   *
   * @param string $command
   *   The command. This will be passed to escapeshellcmd().
   * @param array $command_arguments
   *   The command arguments. They will have to be manually passed to
   *   escapeshellarg() if needed.
   * @param string $command_prefix
   *   A prefix to the command. Will no go through escapeshellcmd().
   *   You must had a space at the end because by default there is no space
   *   between $command_prefix and $command when concatenating.
   *
   * @return string|null
   *   The shell_exec return.
   */
  protected function execExtraction(string $command, array $command_arguments = [], string $command_prefix = '') {
    // UTF-8 multibyte characters will be stripped by escapeshellargs() for
    // the default C-locale.
    // So temporarily set the locale to UTF-8 so that the filepath remains
    // valid.
    /** @var string $backup_locale */
    $backup_locale = setlocale(LC_CTYPE, '0');
    setlocale(LC_CTYPE, 'en_US.UTF-8');

    $cmd = $command_prefix . escapeshellcmd($command) . ' ' . implode(' ', $command_arguments);

    // Restore the locale.
    setlocale(LC_CTYPE, $backup_locale);
    // Support UTF-8 commands.
    // @see http://www.php.net/manual/en/function.shell-exec.php#85095
    shell_exec("LANG=en_US.utf-8");

    return shell_exec($cmd);
  }

}
